#include <windows.h>

1 VERSIONINFO
FILEVERSION 2,1,4,0
PRODUCTVERSION 2,1,4,0
FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
FILEFLAGS 0x1L
#else
FILEFLAGS 0x0L
#endif
FILEOS VOS__WINDOWS32
FILETYPE VFT_DLL
FILESUBTYPE 0x0L
{
    BLOCK "StringFileInfo"
    {
        BLOCK "041103A4"
        {
            VALUE "CompanyName", "project nkf\0"
            VALUE "FileDescription", "Network Kanji Filter\0"
            VALUE "FileVersion", "2.1.4.0 1\0"
            VALUE "InternalName", "nkf32\0"
            VALUE "LegalCopyright", "Copyright (C) 1987, Fujitsu LTD. (Itaru ICHIKAWA); Copyright (C) 2003-2004 T.kaneto\0"
            VALUE "OriginalFilename", "nkf32.c\0"
            VALUE "ProductName", "nkf32.dll\0"
            VALUE "ProductVersion", "2.1.4.0 1\0"
        }
    }
    BLOCK "VarFileInfo"
    {
        VALUE "Translation", 0x411, 932
    }
}
